type lbool = L_TRUE | L_FALSE | L_UNKNOWN;;

let string_of_lbool = function
  | L_TRUE -> "true"
  | L_FALSE -> "false"
  | L_UNKNOWN -> "unknown";;

exception Unknown
let bool_of_lbool = function
  | L_TRUE -> true
  | L_FALSE -> false
  | L_UNKNOWN -> raise Unknown;;

type context
external mk_context : unit -> context = "ml_yices_mk_context";;
external dump_context : context -> unit = "ml_yices_dump_context";;
external check_internal : context -> lbool = "ml_yices_check";;

let check_elapsed = ref 0.;;
let check = Profiling.chronometer check_elapsed check_internal;;

external push : context -> unit = "ml_yices_push";;
external pop : context -> unit = "ml_yices_pop";;

type yices_type
external mk_type : context -> string -> yices_type = "ml_yices_mk_type";;

type var_decl
type expr
external mk_var_decl : context -> string -> yices_type -> var_decl = "ml_yices_mk_var_decl";;
external mk_bool_var_decl : context -> string -> var_decl = "ml_yices_mk_bool_var_decl";;
external mk_var_from_decl : context -> var_decl -> expr = "ml_yices_mk_var_from_decl";;

let mk_real_var_decl context name =
  mk_var_decl context name (mk_type context "real");;

let mk_real_var context name =
  mk_var_from_decl context (mk_real_var_decl context name);;

external mk_true : context -> expr = "ml_yices_mk_true";;
external mk_false : context -> expr = "ml_yices_mk_false";;
external assert_expr : context -> expr -> unit = "ml_yices_assert_expr";;
external mk_bool_var : context -> string -> expr = "ml_yices_mk_bool_var";;
external mk_fresh_bool_var : context -> expr = "ml_yices_mk_fresh_bool_var";;
external mk_and : context -> expr list -> expr = "ml_yices_mk_and";;
external mk_or : context -> expr list -> expr = "ml_yices_mk_or";;
external mk_sum : context -> expr list -> expr = "ml_yices_mk_sum";;
external mk_sub : context -> expr list -> expr = "ml_yices_mk_sub";;
external mk_mul : context -> expr list -> expr = "ml_yices_mk_mul";;
external mk_lt : context -> expr -> expr -> expr = "ml_yices_mk_lt";;
external mk_le : context -> expr -> expr -> expr = "ml_yices_mk_le";;
external mk_gt : context -> expr -> expr -> expr = "ml_yices_mk_gt";;
external mk_ge : context -> expr -> expr -> expr = "ml_yices_mk_ge";;
external mk_eq : context -> expr -> expr -> expr = "ml_yices_mk_eq";;
external mk_diseq : context -> expr -> expr -> expr = "ml_yices_mk_diseq";;
external mk_ite : context -> expr -> expr -> expr -> expr = "ml_yices_mk_ite";;
external mk_not : context -> expr -> expr = "ml_yices_mk_not";;

external mk_num : context -> int -> expr = "ml_yices_mk_num";;
external mk_num_from_string : context -> string -> expr = "ml_yices_mk_num_from_mpz";;
external mk_num_from_mpz : context -> Z.t -> expr = "ml_yices_mk_num_from_mpz";;

type model;;
external get_model : context -> model = "ml_yices_get_model";;
external get_value : model -> var_decl -> lbool = "ml_yices_get_value";;
external get_mpq_value : model -> var_decl -> Q.t = "ml_yices_get_mpq_value";;
